
// compressor utility for Commodore Plus/4 programs
// Copyright (C) 2007-2016 Istvan Varga <istvanv@users.sourceforge.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// The Plus/4 program files generated by this utility are not covered by the
// GNU General Public License, and can be used, modified, and distributed
// without any restrictions.

#ifndef P4COMPRESS_DECOMPRESS2_HPP
#define P4COMPRESS_DECOMPRESS2_HPP

#include "plus4emu.hpp"
#include "compress.hpp"
#include <vector>

namespace Plus4Compress {

  class Decompressor_M2 : public Decompressor {
   private:
    static const unsigned char  sfxModuleLibrary[2984];
    unsigned int  lengthDecodeTable[8 * 2];
    unsigned int  offs1DecodeTable[4 * 2];
    unsigned int  offs2DecodeTable[8 * 2];
    unsigned int  offs3DecodeTable[32 * 2];
    size_t        offs3PrefixSize;
    unsigned char shiftRegister;
    int           shiftRegisterCnt;
    const unsigned char *inputBuffer;
    size_t        inputBufferSize;
    size_t        inputBufferPosition;
    // --------
    unsigned int readBits(size_t nBits);
    unsigned char readLiteralByte();
    // returns LZ match length (1..65535), or zero for literal byte,
    // or length + 0x80000000 for literal sequence
    unsigned int readMatchLength();
    unsigned int readLZMatchParameter(unsigned char slotNum,
                                      const unsigned int *decodeTable);
    void readDecodeTables();
    bool decompressDataBlock(std::vector< unsigned char >& buf,
                             std::vector< bool >& bytesUsed);
   public:
    Decompressor_M2();
    virtual ~Decompressor_M2();
    virtual void decompressData(
        std::vector< std::vector< unsigned char > >& outBuf,
        const std::vector< unsigned char >& inBuf);
    // 'runAddr' can be the start address (0x0000 to 0xFFFF) of a machine code
    // program, or -1 for running a basic program, -2 to return to basic,
    // or -3 for monitor
    static void getSFXModule(std::vector< unsigned char >& outBuf,
                             int runAddr = -2, bool c16Mode = false,
                             bool noCRCCheck = false, bool noReadBuffer = false,
                             int borderEffectType = 2,
                             bool noBlankDisplay = false,
                             bool noZPSaveRestore = false,
                             bool noROM = false, bool noCLI = false);
  };

}       // namespace Plus4Compress

#endif  // P4COMPRESS_DECOMPRESS2_HPP

